@@ -1,22 +1,19 @@
 """
 pyrad.io.trajectory
 ===================
-
 Trajectory class implementation for reading trajectory file.
 Converting to different coordinate systems.
-
 .. autosummary::
     :toctree: generated/
-
     Trajectory
     _Radar_Trajectory
-
 """
 
 import sys
 import re
 import datetime
 import locale
+import pandas as pd
 from warnings import warn
 from copy import deepcopy
 
@@ -31,7 +28,6 @@ from ..io.read_data_sensor import read_trt_thundertracking_traj_data
 class Trajectory(object):
     """
     A class for reading and handling trajectory data from a file.
-
     Attributes
     ----------
     filename : str
@@ -68,8 +64,6 @@ class Trajectory(object):
           For 'lightning' only. Flash number of each data sample
     dBm : array of floats
           For 'lightning' only. Lightning power (dBm)
-
-
     Methods:
     --------
     add_radar : Add a radar
@@ -79,17 +73,16 @@ class Trajectory(object):
     get_samples_in_period : Get indices of samples within period
     _convert_traj_to_swissgrid : convert data from WGS84 to Swiss coordinates
     _read_traj : Read plane trajectory from file
+    _read_traj_flores : Read FLORAKO plane trajectory from file
     _read_traj_trt : Read TRT trajectory from file
     _read_traj_lightning : Read lightning trajectory from file
     _get_total_seconds : Get the total time of the trajectory in seconds
-
     """
 
     def __init__(self, filename, starttime=None, endtime=None,
                  trajtype='plane', flashnr=0):
         """
         Initalize the object.
-
         Parameters
         ----------
         filename : str
@@ -138,6 +131,8 @@ class Trajectory(object):
                 self._read_traj_lightning(flashnr)
             elif self.trajtype == 'trt':
                 self._read_traj_trt()
+            elif self.trajtype == 'plane_flores':
+                self._read_traj_flores()
             else:
                 self._read_traj()
 
@@ -151,16 +146,13 @@ class Trajectory(object):
         """
         Add the coordinates (WGS84 longitude, latitude and non WGS84 altitude)
         of a radar to the radar_list.
-
         Parameters
         ----------
         radar : pyart radar object
             containing the radar coordinates
-
         Return
         ------
         Radar object
-
         """
 
         # Check if radar location is already in the radar list
@@ -195,7 +187,6 @@ class Trajectory(object):
     def calculate_velocities(self, radar):
         """
         Calculate velocities.
-
         """
 
         if not radar.traj_assigned:
@@ -248,7 +239,6 @@ class Trajectory(object):
     def get_start_time(self):
         """
         Get time of first trajectory sample.
-
         Return
         ------
         datetime object
@@ -259,7 +249,6 @@ class Trajectory(object):
     def get_end_time(self):
         """
         Get time of last trajectory sample.
-
         Return
         ------
         datetime object
@@ -284,7 +273,6 @@ class Trajectory(object):
     def _read_traj(self):
         """
         Read trajectory from file
-
         File format
         -----------
         Comment symbol: '#'
@@ -294,7 +282,6 @@ class Trajectory(object):
         3. WGS84 latitude [radians]
         4. WGS84 longitude [radians]
         5. WGS84 altitude [m]
-
         """
 
         # check if the file can be read
@@ -382,11 +369,55 @@ class Trajectory(object):
                 locale.setlocale(locale.LC_ALL, loc)  # restore saved locale
 
         self.nsamples = len(self.time_vector)
-
+        
+    def _read_traj_flores(self):
+        """
+        Read trajectory from FLORAKO file
+        File format
+        -----------
+        Column/Variable Contents, Units and Description:
+        01: UTCDate      Year-Month-Day                 Date of epoch or feature (UTC time)
+        02: UTCTime      HH:MM:SS.SS                    Time of epoch or feature - Receiver time frame (UTC)
+        03: Latitude     Decimal Degrees (signed)       North/South Geographic coordinate
+        04: Longitude    Decimal Degrees (signed)       East/West Geographic coordinate
+        05: H-MSL        Metres                         Height above the geoid
+        06: H-Ell        Metres                         Height above the current ellipsoid
+        07: Undulation   Metres                         Height of the ellipsoid above the geoid
+        08: PDOP                                        Position Dilution of Precision, which is a measure of X, Y, Z position geometry
+        09: HDOP                                        Horizontal Position Dilution of Precision, which is a measure of X, Y position geometry
+        10: VDOP                                        Vertical Position Dilution of Precision, which is a measure of Z position geometry
+        11: DopRms                                      Root mean square of all L1 Doppler measurement residuals
+        """
+          # check if the file can be read
+        try:
+            tfile = open(self.filename, "r")
+        except:
+            raise Exception("ERROR: Could not find|open trajectory file '" +
+                            self.filename+"'")
+        tfile.close()
+        
+        try:
+            data = pd.read_csv(self.filename, skiprows=28, 
+                               delim_whitespace=True)
+            data = data.drop(0)
+            times = [datetime.datetime.strptime(d+'_'+h+'0000',
+                                                '%Y-%m-%d_%H:%M:%S.%f') for 
+                                                d, h in zip(data['UTCDate'],
+                                                            data['UTCTime'])]
+                       
+            self.time_vector = np.array(times)
+            self.wgs84_lat_deg = np.array(pd.to_numeric(data['Latitude']))
+            self.wgs84_lon_deg = np.array(pd.to_numeric(data['Longitude']))
+            self.wgs84_alt_m = np.array(pd.to_numeric(data['H-MSL']))
+                
+        except:
+            raise
+            
+        self.nsamples = len(self.time_vector)
+        
     def _read_traj_lightning(self, flashnr=0):
         """
         Read trajectory from lightning file
-
         File format
         -----------
         Columns:
@@ -397,12 +428,10 @@ class Trajectory(object):
         4. WGS84 longitude [deg]
         5. WGS84 altitude [m]
         6. Power [dBm]
-
         Parameters
         ----------
         flashnr : int
             the flash number to keep. If 0 data from all flashes will be kept
-
         """
         flashnr_vec, time, time_in_flash, lat, lon, alt, dBm = read_lightning(
             self.filename)
@@ -456,7 +485,6 @@ class Trajectory(object):
     def _read_traj_trt(self):
         """
         Read trajectory from TRT file
-
         File format
         -----------
         Columns:
@@ -488,10 +516,8 @@ class Trajectory(object):
         25. Dvel_x
         26. Dvel_y
         27. cell_contours
-
         Parameters
         ----------
-
         """
         if '_tt.trt' in self.filename:
             (traj_ID, _, yyyymmddHHMM, _, _, _, lon, lat, _, _, _, _, _, _, _,
@@ -562,7 +588,6 @@ class Trajectory(object):
 class _Radar_Trajectory:
     """
     A class for holding the trajectory data assigned to a radar.
-
     Attributes
     ----------
     latitude : float
@@ -582,7 +607,6 @@ class _Radar_Trajectory:
     v_abs, v_r, v_el, v_az : array-like
        Velocity vectors of the absolute [m/s], radial [m/s], elevation [deg/s]
        and azimuth [deg/s] velocities
-
     Methods:
     --------
     location_is_equal
@@ -594,7 +618,6 @@ class _Radar_Trajectory:
     def __init__(self, lat, lon, alt):
         """
         Initalize the object.
-
         Parameters
         ----------
         lat, lon , alt : radar location coordinates
@@ -628,11 +651,9 @@ class _Radar_Trajectory:
     def location_is_equal(self, lat, lon, alt):
         """
         Check if the given coordinates are the same.
-
         Parameters
         ----------
         lat, lon , alt : radar location coordinates
-
         Return
         ------
         True if the radar location is equal, False otherwise
@@ -653,7 +674,6 @@ class _Radar_Trajectory:
         """
         Assign a trajectory to the radar in polar radar
         coordinates.
-
         Parameters
         ----------
         el, az, rr : array-like
@@ -673,7 +693,6 @@ class _Radar_Trajectory:
         """
         Convert the radar location (in WGS84 coordinates) to
         swiss CH1903 coordinates.
-
         """
 
         if self._swiss_coords_done:
@@ -689,7 +708,6 @@ class _Radar_Trajectory:
     def assign_velocity_vecs(self, v_abs, v_r, v_el, v_az):
         """
         Assign velocity vectors to the radar.
-
         """
 
         if self._velocity_vecs_assigned:
