@@ -10,6 +10,7 @@ Functions for echo classification and filtering
     process_echo_id
     process_birds_id
     process_clt_to_echo_id
+    process_hydro_mf_to_echo_id
     process_hydro_mf_to_hydro
     process_echo_filter
     process_cdf
@@ -284,6 +285,70 @@ def process_clt_to_echo_id(procstatus, dscfg, radar_list=None):
     return new_dataset, ind_rad
 
 
+def process_hydro_mf_to_echo_id(procstatus, dscfg, radar_list=None):
+    """
+    Converts MF hydrometeor classification into pyrad echo ID
+
+    Parameters
+    ----------
+    procstatus : int
+        Processing status: 0 initializing, 1 processing volume,
+        2 post-processing
+    dscfg : dictionary of dictionaries
+        data set configuration. Accepted Configuration Keywords::
+
+        datatype : list of string. Dataset keyword
+            The input data types
+    radar_list : list of Radar objects
+        Optional. list of radar objects
+
+    Returns
+    -------
+    new_dataset : dict
+        dictionary containing the output
+    ind_rad : int
+        radar index
+
+    """
+
+    if procstatus != 1:
+        return None, None
+
+    for datatypedescr in dscfg['datatype']:
+        radarnr, _, datatype, _, _ = get_datatype_fields(datatypedescr)
+        if datatype == 'hydroMF':
+            clt_field = get_fieldname_pyart(datatype)
+            break
+
+    ind_rad = int(radarnr[5:8])-1
+    if radar_list[ind_rad] is None:
+        warn('No valid radar')
+        return None, None
+    radar = radar_list[ind_rad]
+
+    if clt_field not in radar.fields:
+        warn('MF hydrometeor classification not present.'
+             ' Unable to obtain echoID')
+        return None, None
+
+    echo_id = np.zeros((radar.nrays, radar.ngates), dtype=np.uint8)+1
+    clt = radar.fields[clt_field]['data']
+    echo_id[clt >= 8] = 3  # precip
+    echo_id[np.logical_and(clt < 8, clt > 1)] = 2  # clt
+    echo_id[np.ma.getmaskarray(clt)] = 1  # noise
+    echo_id[clt == 1] = 1  # missing Zh
+
+    id_field = pyart.config.get_metadata('radar_echo_id')
+    id_field['data'] = echo_id
+
+    # prepare for exit
+    new_dataset = {'radar_out': deepcopy(radar)}
+    new_dataset['radar_out'].fields = dict()
+    new_dataset['radar_out'].add_field('radar_echo_id', id_field)
+
+    return new_dataset, ind_rad
+
+
 def process_hydro_mf_to_hydro(procstatus, dscfg, radar_list=None):
     """
     Converts the hydrometeor classification from Météo France to
@@ -1269,11 +1334,11 @@ def process_centroids(procstatus, dscfg, radar_list=None):
         pdf_zh_max : int
             Multiplicative factor to the Guassian function used to make the
             distribution of the reflectivity platykurtic that determines the
-            number of samples for each bin. Default 20000
+            number of samples for each bin. Default 10000
         pdf_relh_max : int
             Multiplicative factor to the Guassian function used to make the
             distribution of the height relative to the iso-0 platykurtic that
-            determines the number of samples for each bin. Default 10000
+            determines the number of samples for each bin. Default 20000
         sigma_zh, sigma_relh : float
             sigma of the respective Gaussian functions. Defaults 0.75 and 1.5
         randomize : bool
@@ -1294,6 +1359,9 @@ def process_centroids(procstatus, dscfg, radar_list=None):
             class. Default 30
         internal_iterations : int. Dataset keyword
             Maximum number of iterations of the internal loop. Default 10
+        sample_data : Bool.
+            If True the data is going to be sampled prior to each external
+            iteration. Default False
         nsamples_iter : int.
             Number of samples per iteration. Default 20000
         alpha : float
@@ -1328,15 +1396,29 @@ def process_centroids(procstatus, dscfg, radar_list=None):
         parallelized : bool
             If True the centroids search is going to be parallelized. Default
             False
-        sample_data : bool
-            If True the data is going to be sampled prior to each external
-            iteration. Default True
         kmax_iter : int
             Maximum number of iterations of the k-medoids algorithm. Default
             100
+        nsamples_small : int
+            Maximum number before using the k-medoids CLARA algorithm. If this
+            number is exceeded the CLARA algorithm will be used. Default 40000
+        sampling_size_clara : int
+            Number of samples used in each iteration of the k-medoids CLARA
+            algorithm. Default 10000
+        niter_clara : int
+            Number of iterations performed by the k-medoids CLARA algorithm.
+            Default 5
         keep_labeled_data : bool
             If True the labeled data is going to be kept for storage. Default
             True
+        use_median : bool
+            If True the intermediate centroids are computed as the median
+            of the observation variables and the final centroids are computed
+            as the median of the intermediate centroids. If false they are
+            computed using the kmedoids algorithm. Default false
+        allow_label_duplicates : bool
+            If True allow to label multiple clusters with the same label.
+            Default True
 
     radar_list : list of Radar objects
         Optional. list of radar objects
@@ -1483,8 +1565,8 @@ def process_centroids(procstatus, dscfg, radar_list=None):
 
     # select data to be used to determine centroids
     nbins = dscfg.get('nbins', 110)
-    pdf_zh_max = dscfg.get('pdf_zh_max', 2500)
-    pdf_relh_max = dscfg.get('pdf_relh_max', 1250)
+    pdf_zh_max = dscfg.get('pdf_zh_max', 10000)
+    pdf_relh_max = dscfg.get('pdf_relh_max', 20000)
     sigma_zh = dscfg.get('sigma_zh', 0.75)
     sigma_relh = dscfg.get('sigma_relh', 1.5)
     randomize = dscfg.get('randomize', True)
@@ -1515,23 +1597,28 @@ def process_centroids(procstatus, dscfg, radar_list=None):
     relh_slope = dscfg.get('relh_slope', 0.001)
     external_iterations = dscfg.get('external_iterations', 30)
     internal_iterations = dscfg.get('internal_iterations', 10)
-    nmedoids_min = dscfg.get('nmedoids_min', 1)
+    sample_data = dscfg.get('sample_data', False)
     nsamples_iter = dscfg.get('nsamples_iter', 20000)
     alpha = dscfg.get('alpha', 0.01)
     cv_approach = dscfg.get('cv_approach', True)
     n_samples_syn = dscfg.get('nsamples_syn', 50)
-    num_samples_arr = dscfg.get('nsamples_syn', (30, 35, 40))
+    num_samples_arr = dscfg.get('num_samples_arr', (30, 35, 40))
     acceptance_threshold = dscfg.get('acceptance_threshold', 0.5)
+    nmedoids_min = dscfg.get('nmedoids_min', 1)
     var_names = dscfg.get(
         'var_names', ('dBZ', 'ZDR', 'KDP', 'RhoHV', 'H_ISO0'))
     hydro_names = dscfg.get(
         'hydro_names',
         ('AG', 'CR', 'LR', 'RP', 'RN', 'VI', 'WS', 'MH', 'IH/HDG'))
-    weight = dscfg.get('weight', (1., 1., 1., 1., 0.75))
+    weight = dscfg.get('weight', (1., 1., 1., 1., 1.))
     parallelized = dscfg.get('parallelized', False)
-    sample_data = dscfg.get('sample_data', True)
     kmax_iter = dscfg.get('kmax_iter', 100)
+    nsamples_small = dscfg.get('nsamples_small', 40000)
+    sampling_size_clara = dscfg.get('sampling_size_clara', 10000)
+    niter_clara = dscfg.get('niter_clara', 5)
     keep_labeled_data = dscfg.get('keep_labeled_data', True)
+    use_median = dscfg.get('use_median', True)
+    allow_label_duplicates = dscfg.get('allow_label_duplicates', True)
 
     (labeled_data, labels, medoids_dict,
      final_medoids_dict) = pyart.retrieve.compute_centroids(
@@ -1544,7 +1631,10 @@ def process_centroids(procstatus, dscfg, radar_list=None):
         acceptance_threshold=acceptance_threshold,
         band=dscfg['global_data']['band'], relh_slope=relh_slope,
         parallelized=parallelized, sample_data=sample_data,
-        kmax_iter=kmax_iter, keep_labeled_data=keep_labeled_data)
+        kmax_iter=kmax_iter, nsamples_small=nsamples_small,
+        sampling_size_clara=sampling_size_clara,
+        niter_clara=niter_clara, keep_labeled_data=keep_labeled_data,
+        use_median=use_median, allow_label_duplicates=allow_label_duplicates)
 
     if not medoids_dict:
         return new_dataset, ind_rad
@@ -1767,6 +1857,46 @@ def process_melting_layer(procstatus, dscfg, radar_list=None):
             max_length_holes=max_length_holes,
             check_min_length=check_min_length, get_iso0=get_iso0)
 
+    elif dscfg['ML_METHOD'] == 'MF':
+        for datatypedescr in dscfg['datatype']:
+            radarnr, _, datatype, _, _ = get_datatype_fields(datatypedescr)
+            if datatype == 'dBZ':
+                refl_field = 'reflectivity'
+            if datatype == 'dBZc':
+                refl_field = 'corrected_reflectivity'
+            if datatype == 'RhoHV':
+                rhohv_field = 'cross_correlation_ratio'
+            if datatype == 'RhoHVc':
+                rhohv_field = 'corrected_cross_correlation_ratio'
+
+        ind_rad = int(radarnr[5:8])-1
+        if radar_list[ind_rad] is None:
+            warn('No valid radar')
+            return None, None
+        radar = radar_list[ind_rad]
+
+        if ((refl_field not in radar.fields) or
+                (rhohv_field not in radar.fields)):
+            warn('Unable to detect melting layer. Missing data')
+            return None, None
+
+        # User defined parameters
+        max_range = dscfg.get('max_range', 20000.)
+        detect_threshold = dscfg.get('detect_threshold', 0.02)
+        interp_holes = dscfg.get('interp_holes', False)
+        max_length_holes = dscfg.get('max_length_holes', 250)
+        check_min_length = dscfg.get('check_min_length', True)
+        get_iso0 = dscfg.get('get_iso0', True)
+
+        ml_obj, ml_dict, iso0_dict, _ = pyart.retrieve.melting_layer_mf(
+            radar, refl_field=refl_field, rhohv_field=rhohv_field,
+            ml_field='melting_layer', ml_pos_field='melting_layer_height',
+            iso0_field='height_over_iso0', max_range=max_range,
+            detect_threshold=detect_threshold, interp_holes=interp_holes,
+            max_length_holes=max_length_holes,
+            check_min_length=check_min_length, get_iso0=get_iso0)
+
+
     elif dscfg['ML_METHOD'] == 'FROM_HYDROCLASS':
         for datatypedescr in dscfg['datatype']:
             radarnr, _, datatype, _, _ = get_datatype_fields(datatypedescr)
